"use strict";

var functions = require('./Functions');
var logger = require('./logger');

/**
 * This object contains methods that will communicate with server.
 */
var ServerConnector = {};

/**
 * List of {@link OverlayCollection} that was added to the server connector. It
 * describes list of all overlays that communicate with server.
 */
ServerConnector._overlays = [];

/**
 * Associative array with methods that should be used for comunication of the
 * {@link OverlayCollection} from _overlays field.
 */
ServerConnector._overlayMethods = [];

/**
 * Form where fields passed to/from server are stored.
 */
ServerConnector.formIdentifier = "_gmapForm";

/**
 * Variable describing when was last transmition of data (location of the map,
 * zoom level, etc.) to the server session.
 */
ServerConnector.lastActualization = 0;

ServerConnector._customMap = null;

/**
 * Number of requests send to server about AliasLayout data.
 */
ServerConnector._waitingForAliasLayoutData = 0;
ServerConnector._waitingForAliasLayoutDataByLayoutId = [];

ServerConnector._waitingFullAliasLayoutData = 0;
ServerConnector._waitingFullAliasLayoutDataById = [];

/**
 * Number of requests send to server about AliasData data.
 */
ServerConnector._waitingForAliasData = 0;
ServerConnector._waitingForAliasDataByAliasId = [];

/**
 * Number of requests send to server about ReactionLayout data.
 */
ServerConnector._waitingForReactionLayoutData = 0;
ServerConnector._waitingForReactionLayoutDataByLayoutId = [];

/**
 * Number of requests send to server about ReactionData data.
 */
ServerConnector._waitingForReactionData = 0;
ServerConnector._waitingForReactionDataByReactionId = [];

/**
 * Get name of the file that should be used as logo.
 */
ServerConnector.getLogoImg = function() {
  return document.getElementById('logoImg').value;
};

/**
 * Get link to the website that should be accessed when clicking on
 * {@link ServerConnector.getLogoImg}.
 */
ServerConnector.getLogoLink = function() {
  return document.getElementById('logoLink').value;
};

/**
 * Get zoom level of the {@link CustomMap} at which it was last browsed.
 */
ServerConnector.getZoomLevel = function() {
  return document.getElementById(ServerConnector.formIdentifier + ':zoomLevel').value;
};

/**
 * Set zoom level for {@link CustomMap} on the server side.
 */
ServerConnector.setZoomLevel = function(value) {
  document.getElementById(ServerConnector.formIdentifier + ':zoomLevel').value = ""
      + value;
};

/**
 * Set x coordinate for {@link CustomMap} on the server side.
 */
ServerConnector.setCenterCoordinateX = function(value) {
  document
      .getElementById(ServerConnector.formIdentifier + ':centerCoordinateX').value = ""
      + value;
};

/**
 * Get x coordinate of the {@link CustomMap} at which it was last browsed.
 */
ServerConnector.getCenterCoordinateX = function() {
  var result = parseFloat(document
      .getElementById(ServerConnector.formIdentifier + ':centerCoordinateX').value);
  return result;
};

/**
 * Set y coordinate for {@link CustomMap} on the server side.
 */
ServerConnector.setCenterCoordinateY = function(value) {
  document
      .getElementById(ServerConnector.formIdentifier + ':centerCoordinateY').value = ""
      + value;
};

/**
 * Get y coordinate of the {@link CustomMap} at which it was last browsed.
 */
ServerConnector.getCenterCoordinateY = function() {
  var result = parseFloat(document
      .getElementById(ServerConnector.formIdentifier + ':centerCoordinateY').value);
  return result;
};

/**
 * Set layout that is currently browsed for {@link CustomMap} on the server
 * side.
 */
ServerConnector.setSelectedLayout = function(value) {
  document.getElementById(ServerConnector.formIdentifier + ':selectedLayout').value = ""
      + value;
};
/**
 * Sets list of layouts visualized by javascript.
 */
ServerConnector.setVisibleLayouts = function(value) {
  document.getElementById(ServerConnector.formIdentifier + ':visibleLayouts').value = value;
};

/**
 * Get layout of the {@link CustomMap} which was last browsed.
 */
ServerConnector.getSelectedLayout = function() {
  return document.getElementById(ServerConnector.formIdentifier
      + ':selectedLayout').value;
};

/**
 * Sends data about coordinates/zoom level etc. to the server. Method checks if
 * the transmition is not done to often (at most once per second)
 */
ServerConnector.actualizeSessionData = function() {
  var timestamp = new Date().getTime();
  // we send the data with at least 1 second time distance
  if (timestamp > ServerConnector.lastActualization) {
    ServerConnector.lastActualization = timestamp + 1000;
    setTimeout(function() {
      ServerConnector.actualizeParams();
    }, 1000);
  }
};

/**
 * Sends data about coordinates/zoom level etc. to the server. Sending is
 * performed immediatelly.
 * 
 * @see ServerConnector.actualizeSessionData
 */
ServerConnector.actualizeParams = function() {
  actualizeParams();
};

/**
 * Sets search query that will be handled by server.
 */
ServerConnector.setSearchQuery = function(value) {
  document.getElementById(ServerConnector.formIdentifier + ':mapParam').value = ""
      + value;
};

/**
 * Sets data mining query that will be handled by server.
 */
ServerConnector.setDataMiningQuery = function(value) {
  document.getElementById(ServerConnector.formIdentifier
      + ':missingConnectionParam').value = "" + value;
};

/**
 * Sets drug search query that will be handled by server.
 */
ServerConnector.setDrugQuery = function(value) {
  document.getElementById(ServerConnector.formIdentifier + ':drugParam').value = ""
      + value;
};

/**
 * Sets comment query that will be handled by server.
 */
ServerConnector.setCommentQuery = function(value) {
  logger.error("ServerConnector.setCommentQuery: Not implemented");
  document.getElementById(ServerConnector.formIdentifier + ':commentParam').value = ""
      + value;
};

/**
 * Sets selected polygon. This might be in futer used oto extract/analyze data
 * in the polygon by the server side.
 */
ServerConnector.setSelectedPolygon = function(value) {
  document.getElementById(ServerConnector.formIdentifier + ':selectedPolygon').value = ""
      + value;
  document.getElementById(ServerConnector.formIdentifier
      + ':selectedDrugPolygon').value = "" + value;
};

/**
 * Sets identifier of the model where selected polygon (@see
 * ServerConnector.setSelectedPolygon) was selected.
 */
ServerConnector.setExportModelId = function(value) {
  document.getElementById(ServerConnector.formIdentifier + ':exportModelId').value = ""
      + value;
};

/**
 * Sets flag marking if the comments are visible or not.
 */
ServerConnector.setShowComments = function(value) {
  _setShowComments([ {
    name : 'showComments',
    value : value
  } ]);
};

/**
 * Search for similar data mining entries on the map.
 */
ServerConnector.showSimilarDataMining = function(param) {
  document.getElementById(ServerConnector.formIdentifier + ':geneName').value = ""
      + param;
  searchMissingConnections();
};

/**
 * Ask server to send list of aliases that were highlighted in the layout with a
 * given identifier.
 * 
 * @param identifier
 *            of the layout
 */
ServerConnector.retreiveActiveAliasesForLayout = function(identifier) {
  this.setWaitingForAliasLayoutData(parseInt(identifier), true);
  _retreiveActiveAliasesForLayout([ {
    name : 'layoutId',
    value : identifier
  } ]);
};

ServerConnector.retreiveFullAliasesForLayout = function(layoutId, aliasId,
    modelId) {
  this.setWaitingForFullAliasLayoutData(parseInt(layoutId), parseInt(aliasId),
      true);
  _retreiveFullAliasesForLayout([ {
    name : 'layoutId',
    value : layoutId,
  }, {
    name : 'ids',
    value : JSON.stringify([ [ modelId, aliasId ] ]),
  }, ]);
};

/**
 * Ask server to send list of reactions that were highlighted in the layout with
 * a given identifier.
 * 
 * @param identifier
 *            of the layout
 */
ServerConnector.retreiveActiveReactionsForLayout = function(identifier) {
  this.setWaitingForReactionLayoutData(parseInt(identifier), true);
  _retreiveActiveReactionsForLayout([ {
    name : 'layoutId',
    value : identifier
  } ]);
};

/**
 * Ask server to send list of light information about aliases.
 * 
 * @param ids
 *            list of alias identifiers, every identifier is an array with two
 *            values: modelId, aliasId
 */
ServerConnector.retreiveLightAliases = function(ids) {
  for (var i = 0; i < ids.length; i++) {
    this.setWaitingForAliasData(parseInt(ids[i][1]), true);
  }
  _retreiveLightAliases([ {
    name : 'ids',
    value : JSON.stringify(ids)
  } ]);
};

/**
 * Ask server to send list of full information about aliases.
 * 
 * @param ids
 *            list of alias identifiers, every identifier is an array with two
 *            values: modelId, aliasId
 */
ServerConnector.retreiveFullAliases = function(ids) {
  _retreiveFullAliases([ {
    name : 'ids',
    value : JSON.stringify(ids)
  } ]);
};

/**
 * Ask server to send list of light information about reactions.
 * 
 * @param ids
 *            list of reaction identifiers, every identifier is an array with
 *            two values: modelId, reactionId
 */
ServerConnector.retreiveLightReactions = function(ids) {
  for (var i = 0; i < ids.length; i++) {
    this.setWaitingForReactionData(parseInt(ids[i][1]), true);
  }
  _retreiveLightReactions([ {
    name : 'ids',
    value : JSON.stringify(ids)
  } ]);
};

/**
 * Sends request to server to refresh data for overlay with a given name.
 * 
 * @param overlayName
 *            name of the overlay
 */
ServerConnector.sendRefreshRequest = function(overlayName) {
  var functions = ServerConnector._overlayMethods[overlayName];
  if (functions != null) {
    var refreshFunction = functions.refreshFunction;
    if (refreshFunction != null) {
      refreshFunction();
    } else {
      logger.warn("Refresh function for " + overlayName + " doesn't exist");
    }
  } else {
    logger.warn("[Refresh function] Unknown overlay: ", overlayName);
  }
};

/**
 * Adds {@link OverlayCollection} to ServerConnector.
 */
ServerConnector.addOverlayCollection = function(overlay) {
  if (ServerConnector._overlayMethods[overlay.name] != null) {
    if (ServerConnector._overlays[overlay.name] != null) {
      logger.warn("Overlay with name " + overlay.name + " already added");
      return;
    }
    ServerConnector._overlays[overlay.name] = overlay;
    if (ServerConnector._overlayMethods[overlay.name].initFunction != null) {
      ServerConnector._overlayMethods[overlay.name].initFunction();
    } else {
      logger.warn("Cannot initialize overlay: " + overlay);
    }
  } else {
    logger.warn("Unknown overlay: ", overlay);
  }
};

/**
 * Method that should be called by server to update data in overlay.
 * 
 * @param overlayName
 *            name of overlay for which the data is sent
 * @param data
 *            new data for overlay
 * @param fitBounds
 *            should the map call fitBounds after update
 */
ServerConnector.updateOverlayCollection = function(overlayName, data, fitBounds) {
  logger.debug("Updating: " + overlayName);
  var overlay = ServerConnector._overlays[overlayName];
  if (overlay != null) {
    overlay.updateOverlays(data, fitBounds);
  } else {
    logger.warn("Overlay " + overlayName
        + " cannot be updated. It doesn't exist");
  }
};

/**
 * Name of the overlay for 'search' overlay.
 */
ServerConnector.SEARCH_OVERLAY_NAME = 'search';

/**
 * Name of the overlay for 'data mining' overlay.
 */
ServerConnector.DATA_MINING_OVERLAY_NAME = 'missingConnection';

/**
 * Name of the overlay for 'drug' overlay.
 */
ServerConnector.DRUG_OVERLAY_NAME = 'drug';

/**
 * Name of the overlay for 'chemical' overlay.
 */
ServerConnector.CHEMICAL_OVERLAY_NAME = 'chemical';

/**
 * Name of the overlay for 'comment' overlay.
 */
ServerConnector.COMMENT_OVERLAY_NAME = 'comment';

/**
 * Name of the overlay for 'mirna' overlay.
 */
ServerConnector.MI_RNA_OVERLAY_NAME = 'mirna';

/**
 * Register 'search' overlay on the server.
 */
ServerConnector.registerSearchOverlay = function() {
  _registerSearchOverlayCollection([ {
    name : "overlayName",
    value : ServerConnector.SEARCH_OVERLAY_NAME,
  } ]);
};

/**
 * Sends request to the server to refresh data in 'search' overlay.
 */
ServerConnector.refreshSearchOverlay = function() {
  _refreshSearchOverlayCollection();
};

/**
 * Clear data related to 'search' overlay..
 */
ServerConnector.clearSearchOverlay = function() {
  _clearSearchOverlayCollection();
};

/**
 * Register 'data mining' overlay on the server.
 */
ServerConnector.registerDataMiningOverlay = function() {
  _registerMissingConnectionOverlayCollection([ {
    name : "overlayName",
    value : ServerConnector.DATA_MINING_OVERLAY_NAME,
  } ]);
};

/**
 * Sends request to the server to refresh data in 'data mining' overlay.
 */
ServerConnector.refreshDataMiningOverlay = function() {
  _refreshMissingConnectionOverlayCollection();
};

/**
 * Clear data related to 'data mining' overlay.
 */
ServerConnector.clearDataMiningOverlay = function() {
  _clearMissingConnectionOverlayCollection();
};

/**
 * Register 'drug' overlay on the server.
 */
ServerConnector.registerDrugOverlay = function() {
  _registerDrugOverlayCollection([ {
    name : "overlayName",
    value : ServerConnector.DRUG_OVERLAY_NAME,
  } ]);
};

ServerConnector.requestDrugDetailDataFunction = function(identfiedElement, id) {
  logger.debug("Requesting drug details: ", identfiedElement, id);
  _requestDrugDetailDataFunction([ {
    name : "searchResultIdentifier",
    value : id,
  }, {
    name : "objectIdentifier",
    value : identfiedElement.getId(),
  }, {
    name : "modelIdentifier",
    value : identfiedElement.getModelId(),
  }, {
    name : "type",
    value : identfiedElement.getType(),
  } ]);
};
ServerConnector.requestCommentDetailDataFunction = function(identfiedElement,
    id) {
  _requestCommentDetailDataFunction([ {
    name : "searchResultIdentifier",
    value : id,
  }, {
    name : "objectIdentifier",
    value : identfiedElement.getId(),
  }, {
    name : "modelIdentifier",
    value : identfiedElement.getModelId(),
  }, {
    name : "type",
    value : identfiedElement.getType(),
  } ]);
};

/**
 * Sends request to the server to refresh data in 'drug' overlay.
 */
ServerConnector.refreshDrugOverlay = function() {
  _refreshDrugOverlayCollection();
};

/**
 * Clear data related to 'drug' overlay.
 */
ServerConnector.clearDrugOverlay = function() {
  _clearDrugOverlayCollection();
};

ServerConnector.registerChemicalOverlay = function() {
  _registerChemicalOverlayCollection([ {
    name : "overlayName",
    value : ServerConnector.CHEMICAL_OVERLAY_NAME,
  } ]);
};

ServerConnector.requestChemicalDetailDataFunction = function(identfiedElement,
    id) {
  _requestChemicalDetailDataFunction([ {
    name : "searchResultIdentifier",
    value : id,
  }, {
    name : "objectIdentifier",
    value : identfiedElement.getId(),
  }, {
    name : "modelIdentifier",
    value : identfiedElement.getModelId(),
  }, {
    name : "type",
    value : identfiedElement.getType(),
  } ]);
};

/**
 * Sends request to the server to refresh data in 'drug' overlay.
 */
ServerConnector.refreshChemicalOverlay = function() {
  _refreshChemicalOverlayCollection();
};

/**
 * Clear data related to 'chemical' overlay.
 */
ServerConnector.clearChemicalOverlay = function() {
  _clearChemicalOverlayCollection();
};

/**
 * Register 'comment' overlay on the server.
 */
ServerConnector.registerCommentOverlay = function() {
  _registerCommentOverlayCollection([ {
    name : "overlayName",
    value : ServerConnector.COMMENT_OVERLAY_NAME,
  } ]);
};

/**
 * Sends request to the server to refresh data in 'comment' overlay.
 */
ServerConnector.refreshCommentOverlay = function() {
  _refreshCommentOverlayCollection();
};

/**
 * Clear data related to 'comment' overlay.
 */
ServerConnector.clearCommentOverlay = function() {
  _clearCommentOverlayCollection();
};

/**
 * Clear data related to 'data mining' overlay.
 */
ServerConnector.clearDataMiningOverlay = function() {
  _clearMissingConnectionOverlayCollection();
};

/**
 * Register 'drug' overlay on the server.
 */
ServerConnector.registerMiRnaOverlay = function() {
  _registerMiRnaOverlayCollection([ {
    name : "overlayName",
    value : ServerConnector.MI_RNA_OVERLAY_NAME,
  } ]);
};

ServerConnector.requestMiRnaDetailDataFunction = function(identfiedElement, id) {
  _requestMiRnaDetailDataFunction([ {
    name : "searchResultIdentifier",
    value : id,
  }, {
    name : "objectIdentifier",
    value : identfiedElement.getId(),
  }, {
    name : "modelIdentifier",
    value : identfiedElement.getModelId(),
  }, {
    name : "type",
    value : identfiedElement.getType(),
  } ]);
};

/**
 * Sends request to the server to refresh data in 'drug' overlay.
 */
ServerConnector.refreshMiRnaOverlay = function() {
  _refreshMiRnaOverlayCollection();
};

/**
 * Clear data related to 'mirna' overlay.
 */
ServerConnector.clearMiRnaOverlay = function() {
  _clearMiRnaOverlayCollection();
};

/**
 * Define pack of methods for 'search' overlay.
 */
ServerConnector._overlayMethods[ServerConnector.SEARCH_OVERLAY_NAME] = {
  initFunction : ServerConnector.registerSearchOverlay,
  refreshFunction : ServerConnector.refreshSearchOverlay,
  clearFunction : ServerConnector.clearSearchOverlay,
};

/**
 * Define pack of methods for 'data minig' overlay.
 */
ServerConnector._overlayMethods[ServerConnector.DATA_MINING_OVERLAY_NAME] = {
  initFunction : ServerConnector.registerDataMiningOverlay,
  refreshFunction : ServerConnector.refreshDataMiningOverlay,
  clearFunction : ServerConnector.clearDataMiningOverlay,
};

/**
 * Define pack of methods for 'drug' overlay.
 */
ServerConnector._overlayMethods[ServerConnector.DRUG_OVERLAY_NAME] = {
  initFunction : ServerConnector.registerDrugOverlay,
  refreshFunction : ServerConnector.refreshDrugOverlay,
  clearFunction : ServerConnector.clearDrugOverlay,
  requestDetailDataFunction : ServerConnector.requestDrugDetailDataFunction,
};

/**
 * Define pack of methods for 'chemical' overlay.
 */
ServerConnector._overlayMethods[ServerConnector.CHEMICAL_OVERLAY_NAME] = {
  initFunction : ServerConnector.registerChemicalOverlay,
  refreshFunction : ServerConnector.refreshChemicalOverlay,
  clearFunction : ServerConnector.clearChemicalOverlay,
  requestDetailDataFunction : ServerConnector.requestChemicalDetailDataFunction,
};

/**
 * Define pack of methods for 'comment' overlay.
 */
ServerConnector._overlayMethods[ServerConnector.COMMENT_OVERLAY_NAME] = {
  initFunction : ServerConnector.registerCommentOverlay,
  refreshFunction : ServerConnector.refreshCommentOverlay,
  clearFunction : ServerConnector.clearCommentOverlay,
  requestDetailDataFunction : ServerConnector.requestCommentDetailDataFunction,
};

ServerConnector._overlayMethods[ServerConnector.MI_RNA_OVERLAY_NAME] = {
  initFunction : ServerConnector.registerMiRnaOverlay,
  refreshFunction : ServerConnector.refreshMiRnaOverlay,
  clearFunction : ServerConnector.clearMiRnaOverlay,
  requestDetailDataFunction : ServerConnector.requestMiRnaDetailDataFunction,
};

ServerConnector.sendClearRequest = function(overlayName) {
  var functions = ServerConnector._overlayMethods[overlayName];
  if (functions != null) {
    var clearFunction = functions.clearFunction;
    if (clearFunction != null) {
      clearFunction();
    } else {
      logger.warn("Clear function for " + overlayName + " doesn't exist");
    }
  } else {
    logger.warn("[Clear function] Unknown overlay: ", overlayName);
  }
};

ServerConnector.setOverlayResultIds = function(overlayName, ids) {
  var overlay = this._overlays[overlayName];
  if (overlay != null) {
    overlay.setResultsIds(ids);
  } else {
    logger.warn("Overlay '" + overlayName + "' doesn't exist");
  }
};

ServerConnector.sendOverlayDetailDataRequest = function(overlayName,
    identifiedElement, general) {
  var overlay = ServerConnector._overlays[overlayName];
  if (overlay == null) {
    logger.warn("Unknown overlay: " + overlayName);
    return;
  }
  var functions = ServerConnector._overlayMethods[overlayName];
  if (functions != null) {
    var requestDetailDataFunction = functions.requestDetailDataFunction;
    if (requestDetailDataFunction != null) {
      var missingData = overlay.getMissingDetailDataIds(identifiedElement,
          general);
      if (missingData.length == 0) {
        logger
            .warn(
                "request of missing data called for empty data set... overlay name = ",
                overlayName, "general: ", general);
      }
      for (var i = 0; i < missingData.length; i++) {
        requestDetailDataFunction(identifiedElement, missingData[i]);
      }
    } else {
      logger.warn("Request detail data function for " + overlayName
          + " doesn't exist");
    }
  } else {
    logger.warn("[Clear function] Unknown overlay: ", overlayName);
  }

};

ServerConnector.updateOverviewElementDetailData = function(overlayName,
    searchResultIdentifier, identifiedElement, data) {
  var overlay = this._overlays[overlayName];
  if (overlay != null) {
    overlay.updateOverviewElementDetailData(identifiedElement,
        searchResultIdentifier, data);
  } else {
    logger.warn("Overlay '" + overlayName + "' doesn't exist");
  }
};

/**
 * Returns <code>true</code> if we are waiting for any kind of data from
 * server, <code>false</code> otherwise.
 * 
 * @returns {Boolean}: <code>true</code> if we are waiting for any kind of
 *          data from server, <code>false</code> otherwise.
 */
ServerConnector.isWaitingForData = function() {
  var result = (this._waitingForAliasLayoutData != 0) || //
  (this._waitingForAliasData != 0) || //
  (this._waitingForReactionLayoutData != 0) || //
  (this._waitingForReactionData != 0);
  return result;
};

/**
 * Sets waitingForAliasLayoutData flag that monitors if we are waiting for the
 * data about aliases in a layout from server.
 * 
 * @param value
 *            new value of the waitingForAliasLayoutData field
 */
ServerConnector.setWaitingForAliasLayoutData = function(layoutId, value) {
  if (functions.isInt(layoutId)) {
    var oldVal = this._waitingForAliasLayoutDataByLayoutId[layoutId];
    this._waitingForAliasLayoutDataByLayoutId[layoutId] = value;
    if (value === true) {
      if (oldVal !== true) {
        this._waitingForAliasLayoutData++;
      }
    } else if (value === false) {
      if (oldVal === true) {
        this._waitingForAliasLayoutData--;
      }
    } else {
      logger
          .error(
              "WaitingForAliasLayoutData value must be boolean, but found: ",
              value);
    }
  } else {
    logger.error(
        "WaitingForAliasLayoutData layoutId must be integer, but found: ",
        layoutId);
  }
}

ServerConnector.setWaitingForFullAliasLayoutData = function(layoutId, aliasId,
    value) {
  if (!functions.isInt(layoutId)) {
    logger
        .error(
            "setWaitingForFullAliasLayoutData layoutId must be integer, but found: ",
            layoutId);
  }
  if (!functions.isInt(aliasId)) {
    logger
        .error(
            "setWaitingForFullAliasLayoutData aliasId must be integer, but found: ",
            aliasId);
  }

  var oldVal = this._waitingFullAliasLayoutDataById[layoutId + "-" + aliasId];
  this._waitingFullAliasLayoutDataById[layoutId + "-" + aliasId] = value;
  if (value === true) {
    if (oldVal !== true) {
      this._waitingFullAliasLayoutData++;
    }
  } else if (value === false) {
    if (oldVal === true) {
      this._waitingFullAliasLayoutData--;
    }
  } else {
    logger.error(
        "setWaitingForFullAliasLayoutData value must be boolean, but found: ",
        value);
  }
}

ServerConnector.isWaitingForFullAliasLayoutData = function(layoutId, aliasId) {
  if (!functions.isInt(layoutId)) {
    logger
        .error(
            "isWaitingForFullAliasLayoutData layoutId must be integer, but found: ",
            layoutId);
  }
  if (!functions.isInt(aliasId)) {
    logger.error(
        "isWaitingForFullAliasLayoutData aliasId must be integer, but found: ",
        aliasId);
  }

  var val = this._waitingFullAliasLayoutDataById[layoutId + "-" + aliasId];
  if (val == null) {
    return false;
  }
  return val;
}

/**
 * Sets waitingForAliasData flag that monitors if we are waiting for the data
 * about aliases from server.
 * 
 * @param value
 *            new value of the waitingForAliasData field
 */
ServerConnector.setWaitingForAliasData = function(aliasId, value) {
  if (functions.isInt(aliasId)) {
    var oldVal = this._waitingForAliasDataByAliasId[aliasId];
    this._waitingForAliasDataByAliasId[aliasId] = value;
    if (value === true) {
      if (oldVal !== true) {
        this._waitingForAliasData++;
      }
    } else if (value === false) {
      if (oldVal === true) {
        this._waitingForAliasData--;
      }
    } else {
      logger.error("WaitingForAliasData value must be boolean, but found: ",
          value);
    }
  } else {
    logger.error("WaitingForAliasData aliasId must be integer, but found: ",
        aliasId);
  }
};

/**
 * Sets waitingForReactionLayoutData flag that monitors if we are waiting for
 * the data about reactions in a layout from server.
 * 
 * @param value
 *            new value of the waitingForReactionLayoutData field
 */
ServerConnector.setWaitingForReactionLayoutData = function(layoutId, value) {
  if (functions.isInt(layoutId)) {
    var oldVal = this._waitingForReactionLayoutDataByLayoutId[layoutId];
    this._waitingForReactionLayoutDataByLayoutId[layoutId] = value;
    if (value === true) {
      if (oldVal !== true) {
        this._waitingForReactionLayoutData++;
      }
    } else if (value === false) {
      if (oldVal === true) {
        this._waitingForReactionLayoutData--;
      }
    } else {
      logger.error(
          "WaitingForReactionLayoutData value must be boolean, but found: ",
          value);
    }
  } else {
    logger.error(
        "WaitingForReactionLayoutData layoutId must be integer, but found: ",
        layoutId);
  }
};
/**
 * Sets waitingForReactionData flag that monitors if we are waiting for the data
 * about reactions from server.
 * 
 * @param value
 *            new value of the waitingForReactionData field
 */
ServerConnector.setWaitingForReactionData = function(reactionId, value) {
  if (functions.isInt(reactionId)) {
    var oldVal = this._waitingForReactionDataByReactionId[reactionId];
    this._waitingForReactionDataByReactionId[reactionId] = value;
    if (value === true) {
      if (oldVal !== true) {
        this._waitingForReactionData++;
      }
    } else if (value === false) {
      if (oldVal === true) {
        this._waitingForReactionData--;
      }
    } else {
      logger.error("WaitingForReactionData value must be boolean, but found: ",
          value);
    }
  } else {
    logger.error(
        "WaitingForReactionData reactionId must be integer, but found: ",
        reactionId);
  }
};

ServerConnector.setCustomMap = function(customMap) {
  this._customMap = customMap;
};

ServerConnector.getCustomMap = function() {
  return this._customMap;
};

ServerConnector.addAliasesForLayout = function(layoutId, jsonAliases) {
  this.setWaitingForAliasLayoutData(parseInt(layoutId), false);
  this.getCustomMap().addAliasesForLayout(layoutId, jsonAliases);
};

ServerConnector.updateAliasesForLayout = function(layoutId, jsonAliases) {
  var arr = JSON.parse(jsonAliases);
  for (var i = 0; i < arr.length; i++) {
    this.setWaitingForFullAliasLayoutData(parseInt(layoutId),
        parseInt(arr[i].idObject), false);
  }
  this.getCustomMap().updateAliasesForLayout(layoutId, jsonAliases);
};

ServerConnector.addAliases = function(aliases) {
  for (var i = 0; i < aliases.length; i++) {
    this.setWaitingForAliasData(parseInt(aliases[i].idObject), false);
  }
  this.getCustomMap().addAliases(aliases);
};

ServerConnector.addReactionsForLayout = function(layoutId, jsonReactions) {
  this.setWaitingForReactionLayoutData(parseInt(layoutId), false);
  this.getCustomMap().addReactionsForLayout(layoutId, jsonReactions);
};

ServerConnector.addReactions = function(jsonReactions) {
  var arr = JSON.parse(jsonReactions);
  for (var i = 0; i < arr.length; i++) {
    this.setWaitingForReactionData(parseInt(arr[i].idObject), false);
  }
  this.getCustomMap().addReactions(jsonReactions);
};

ServerConnector.searchByCoord = function(modelId, latLngCoordinates) {
  _searchByCoord([ {
    name : 'submodelId',
    value : modelId
  }, {
    name : 'latCoord',
    value : latLngCoordinates.lat()
  }, {
    name : 'lngCoord',
    value : latLngCoordinates.lng()
  } ]);
};

ServerConnector.requestUpdateCommentList = function(modelId, latLngCoordinates) {
  _updateCommentList([ {
    name : 'submodelId',
    value : modelId
  }, {
    name : 'latCoord',
    value : latLngCoordinates.lat()
  }, {
    name : 'lngCoord',
    value : latLngCoordinates.lng()
  } ]);
};

ServerConnector.sendReferenceGenomeDetailRequest = function(type, version) {
  logger.debug("Send request", type, version);
  _sendReferenceGenomeDetailRequest([ {
    name : 'type',
    value : type
  }, {
    name : 'version',
    value : version
  } ]);
};
ServerConnector.updateReferenceGenomeData = function(organismId, type, version,
    jsonObj) {
  this.getCustomMap().updateReferenceGenome(type, version, jsonObj);
};

ServerConnector.getMinOverlayColorInt = function() {
  var color = document.getElementById("overlayConfigForm:userMinColor").value;
  if (color == "") {
    color = document.getElementById("overlayConfigForm:systemMinColor").value;
  }
  color = parseInt(color, 16);
  var color = (color & 0xFFFFFF);
  return color;
};

ServerConnector.getMaxOverlayColorInt = function() {
  var color = document.getElementById("overlayConfigForm:userMaxColor").value;
  if (color == "") {
    color = document.getElementById("overlayConfigForm:systemMaxColor").value;
  }
  color = parseInt(color, 16);
  var color = (color & 0xFFFFFF);
  return color;
};

module.exports = ServerConnector;
