"use strict";

var logger = require('../logger');
var AbstractCustomMap = require('./AbstractCustomMap');
var Configuration = require('../Configuration');
var CustomMapOptions = require('./CustomMapOptions');

/**
 * Constructor of a submap. Submaps are created on application start. But dialog
 * (popup window) is initialized on demand using init function.
 * 
 * @param customMap
 *            parent CustomMap
 * @param id
 *            identifier of the submap
 */
function Submap(customMap, id) {
  this.customMap = customMap;
  this.id = id;
  this.configuration = new Configuration();
  this.loadSubmapConfiguration();

  AbstractCustomMap.call(this, new CustomMapOptions({
    map : null,
    configuration : this.configuration,
    hideDiV : customMap.getHideDiv(),
    overviewDiv : customMap.getOverviewDiv(),
    markerOptimization : customMap.isMarkerOptimization(),
    bigLogo : customMap.isBigLogo(),
    customTouchInterface : customMap.isCustomTouchInterface()
  }));

  this.initialized = false;
}

// implementation of object inheritance
Submap.prototype = Object.create(AbstractCustomMap.prototype);
Submap.prototype.constructor = Submap;

/**
 * This method initializes submap with gui component. Before this point submap
 * is created and contains data, but cannot be visualized in the broswer.
 * 
 * @param htmlTag
 *            html div tag where google map should be placed
 * @param jsVar
 *            javascript component of primefaces popup dialog where submap will
 *            be visualized
 */
Submap.prototype.init = function(htmlTag, jsVar) {
  logger.debug("Initializing gui: " + this.id);

  if (jsVar.submapControler != null) {
    throw "Submodel with "
        + id
        + " cannot be created, because provided dialog window already has associated submodel";
  } else {
    this.htmlTag = htmlTag;
    this.jsVar = jsVar;

    this.initialized = true;

    var doc = htmlTag;
    var childDiv = null;
    for (var i = 0; i < doc.childNodes.length; i++) {
      if (doc.childNodes[i].className.indexOf("ui-dialog-content") >= 0) {
        childDiv = doc.childNodes[i];
      }
    }

    var controlDiv = document.createElement('div');
    controlDiv.id = "submap-gmap-div-" + this.getId();
    controlDiv.style.height = '100%';
    controlDiv.style.width = '100%';

    childDiv.appendChild(controlDiv);
    childDiv.style.height = '100%';
    childDiv.style.width = '100%';

    var mapOptions = this.creatMapOptions(configuration.MAPS.length);

    this.map = new google.maps.Map(controlDiv, mapOptions);
    if (this.isCustomTouchInterface()) {
      this._touchInterface = new TouchMap(this);
    }
    this.setupLayouts();

    var self = this;
    self.lastResize = 0;

    jQuery(htmlTag).bind("resize", function() {
      var timestamp = new Date().getTime();
      if (timestamp > self.lastResize) {
        self.lastResize = timestamp + 200;
        setTimeout(function() {
          google.maps.event.trigger(self.map, 'resize');
          self.lastResize = Math.min(new Date().getTime(), self.lastResize);
        }, 100);
      }
    });

    htmlTag.style.width = Math.floor(window.innerWidth * 2 / 3) + "px";
    htmlTag.style.height = Math.floor(window.innerHeight * 2 / 3) + "px";
    google.maps.event.trigger(self.map, 'resize');

    jsVar.submapControler = this;

    this.registerMapClickEvents();

    // after resizing center map
    var centerPoint = new google.maps.LatLng(this.configuration.CENTER_LAT,
        this.configuration.CENTER_LNG);
    self.map.setCenter(centerPoint);

    // and now send the zoom level to the client side
    google.maps.event.addListener(this.map, 'zoom_changed', function() {
      setModelZoomLevel([ {
        name : 'mapId',
        value : self.id
      }, {
        name : 'zoomLevel',
        value : self.map.getZoom()
      } ]);
    });

    setModelZoomLevel([ {
      name : 'mapId',
      value : self.id
    }, {
      name : 'zoomLevel',
      value : self.map.getZoom()
    } ]);
  }

};

Submap.prototype.openLayout = function(identifier) {
  if (this.initialized) {
    this.map.setMapTypeId(identifier);
  }
};

Submap.prototype.loadSubmapConfiguration = function() {
  var self = this;
  var onConfigurationReload = function() {
    var submodelFound = false;
    for (var i = 0; i < self.customMap.configuration.SUBMODELS.length
        && (!submodelFound); i++) {
      if (self.customMap.configuration.SUBMODELS[i].ID_MODEL == self.id) {
        self.configuration = self.customMap.configuration.SUBMODELS[i];
        submodelFound = true;
      }
    }
    if (!submodelFound) {
      throw "Cannot find configuration for submodel " + self.id;
    }
    logger.debug("Submodel config reloaded: " + self.id);
  };

  onConfigurationReload();
  this.customMap.configuration.addListener("onreload", onConfigurationReload);
};

Submap.prototype.getTopMap = function() {
  return this.customMap;
};

module.exports = Submap;
