"use strict";

var logger = require('../../logger');

var Alias = require('./Alias');
var LayoutAlias = require('./LayoutAlias');
var LayoutData = require('./LayoutData');
var LayoutReaction = require('./LayoutReaction');
var PointData = require('./PointData');
var Reaction = require('./Reaction');

// This file describes javascript representation of Java Model class that represents content of the map.

/**
 * Default constructor.
 * 
 * @param configuration
 *            {@link Configuration} used to initialize this map
 */

function MapModel(configuration) {
  // identifier of the map
  this.id = configuration.ID_MODEL;

  // list of aliases is empty (it will be filled dynamically - when necessary)
  this._aliases = [];

  // list of reactions is empty (it will be filled dynamically - when necessary)
  this._reactions = [];

  // list of aliases that should be updated from server side during the next
  // connection
  this._missingAliases = [];

  // list of reactions that should be updated from server side during the next
  // connection
  this._missingReactions = [];

  // list of layouts is empty (it will be filled when necessary)
  this._layoutsData = [];

  // information about points and associated data (for now we have only comments
  // associated to the point,
  // but it can be extended)
  this._pointsData = [];
};

/**
 * Initializes data structure about specific layout.
 * 
 * @param layoutId
 *            identifier of the layout
 */
MapModel.prototype.initLayoutData = function(layoutId, name) {
  var layout = this._layoutsData[layoutId];
  if (layout == null || layout == 'undefined') {
    layout = new LayoutData(layoutId, name);
    this._layoutsData[layoutId] = layout;
  } else {
    logger
        .warn("Layout " + layoutId + " already exists in a model: " + this.id);
  }
};

/**
 * Returns list of {@link LayoutData} on this model.
 * 
 * @returns {Array} with list of {@link LayoutData} on this model
 */
MapModel.prototype.getLayoutsData = function() {
  var result = [];
  for ( var id in this._layoutsData) {
    if (this._layoutsData.hasOwnProperty(id)) {
      result.push(this._layoutsData[id]);
    }
  }
  return result;
};

/**
 * Return list of all aliases that were added to the model.
 */
MapModel.prototype.getAliases = function() {
  var result = [];
  for ( var id in this._aliases) {
    if (this._aliases.hasOwnProperty(id)) {
      result.push(this._aliases[id]);
    }
  }
  return result;
};

/**
 * Returns {@link Alias} by identifier.
 * 
 * @param id
 *            identifier of the {@link Alias}
 * @returns {@link Alias} by identifier
 */
MapModel.prototype.getAliasById = function(id) {
  return this._aliases[id];
};

/**
 * Returns {@link Reaction} by identifier.
 * 
 * @param id
 *            identifier of the {@link Reaction}
 * @returns {@link Reaction} by identifier
 */
MapModel.prototype.getReactionById = function(id) {
  return this._reactions[id];
};

/**
 * Adds information about alias that should be visualized in the specific
 * layout.
 * 
 * @param layoutId
 *            identifier of the layout
 * @param aliasData
 *            raw alias data taken from server side
 */
MapModel.prototype.addAliasForLayout = function(layoutId, aliasData) {
  var layout = this._layoutsData[layoutId];
  var alias = new LayoutAlias(aliasData);
  layout.addAlias(alias);

  //if we don't have basic information about alias then mark alias to be missing
  if (this._aliases[alias.id] == null) {
    if (this._missingAliases[alias.id] == null) {
      this._missingAliases[alias.id] = alias.id;
    }
  }
};

/**
 * Adds information about reaction that should be visualized in the specific
 * layout.
 * 
 * @param layoutId
 *            identifier of the layout
 * @param reactionData
 *            raw reaction data taken from server side
 */
MapModel.prototype.addReactionForLayout = function(layoutId, reactionData) {
  var layout = this.getLayoutDataById(layoutId);
  var reaction = new LayoutReaction(reactionData);
  layout.addReaction(reaction);
  if (this._reactions[reaction.id] == null) {
    if (this._missingReactions[reaction.id] == null) {
      this._missingReactions[reaction.id] = reaction.id;
    }
  }
};

/**
 * Returns layout data for a given layout identifier.
 * 
 * @param layoutId
 *            layout identifier
 * @returns {LayoutData} for a given layout identifier
 */
MapModel.prototype.getLayoutDataById = function(layoutId) {
  return this._layoutsData[layoutId];
};

/**
 * Adds information about alias.
 * 
 * @param aliasData
 *            raw data about alias
 */
MapModel.prototype.addAlias = function(aliasData) {
  var alias = new Alias(aliasData);
  if (this._aliases[alias.id] != null) {
    this._aliases[alias.id].update(alias);
  } else {
    this._aliases[alias.id] = alias;
    if (this._missingAliases[alias.id] != null) {
      this._missingAliases[alias.id] = null;
      delete this._missingAliases[alias.id];
    }
  }
};

/**
 * Adds information about reaction.
 * 
 * @param reactionData
 *            raw data about reaction
 */
MapModel.prototype.addReaction = function(reactionData) {
  var reaction = null;
  if (reactionData instanceof Reaction) {
    reaction = reactionData;
  } else {
    reaction = new Reaction(reactionData);
  }
  if (this._reactions[reaction.id] != null) {
    logger.warn("Reaction with id: " + reaction.id + " already exists");
  } else {
    this._reactions[reaction.id] = reaction;
    if (this._missingReactions[reaction.id] != null) {
      this._missingReactions[reaction.id] = null;
      delete this._missingReactions[reaction.id];
    }
  }
};

/**
 * Returns list of aliases that should be visualized in some layouts, but we
 * have no detailed information about them.
 * 
 * @returns {Array} list of alias identifiers where we miss (and need) detailed
 *          information
 */
MapModel.prototype.getMissingAliasIds = function() {
  var result = [];
  for ( var key in this._missingAliases) {
    result.push(key);
  }
  return result;
};

/**
 * Returns list of reactions that should be visualized in some layouts, but we
 * have no detailed information about them.
 * 
 * @returns {Array} list of reaction identifiers where we miss (and need)
 *          detailed information
 */
MapModel.prototype.getMissingReactionIds = function() {
  var result = [];
  for ( var key in this._missingReactions) {
    result.push(key);
  }
  return result;
};

/**
 * Adds alias to list of missing aliases.
 * 
 * @param id
 *            identifier of the alias that is missing
 */
MapModel.prototype.addMissingAliasId = function(id) {
  if (this._aliases[id] != null) {
    logger.warn("Cannot add alias to list of missing aliases. Alias with id "
        + id + " already loaded.");
  } else {
    this._missingAliases[id] = id;
  }
};

/**
 * Adds reaction to list of missing reactions.
 * 
 * @param id
 *            identifier of the reaction that is missing
 */
MapModel.prototype.addMissingReactionId = function(id) {
  if (this._reactions[id] != null) {
    logger
        .warn("Cannot add reaction to list of missing reactions. Reaction with id "
            + id + " already loaded.");
  } else {
    this._missingReactions[id] = id;
  }
};

/**
 * Returns {@link PointData} for a given point on the map.
 * 
 * @param point
 *            {@link google.maps.Point} where we are requesting data
 * @returns {@link PointData} for a given point on the map
 */
MapModel.prototype.getPointDataByPoint = function(inputPoint) {
  if (inputPoint instanceof google.maps.Point) {
    var point = this._roundPoint(inputPoint);
    var id = this._pointToId(point);
    var result = this._pointsData[id];
    if (result == null) {
      result = new PointData(point, this.id);
      this._pointsData[id] = result;
    }
    return result;
  } else {
    logger.warn("point must be of class: google.maps.Point");
    return null;
  }
};

/**
 * Returns point where x and y coordinate are rounded to 2 decimal places.
 * 
 * @param point
 *            input point
 * @returns {google.maps.Point} point where x and y coordinate are rounded to 2
 *          decimal places
 */
MapModel.prototype._roundPoint = function(point) {
  var x = parseFloat(point.x).toFixed(2);
  var y = parseFloat(point.y).toFixed(2);
  return new google.maps.Point(x, y);
};

/**
 * Transform point into string identifier.
 * 
 * @param point
 *            {google.maps.Point} to transform
 * @returns {String} string identifier for a given point
 */
MapModel.prototype._pointToId = function(point) {
  return "(" + point.x + ", " + point.y + ")";
};

MapModel.prototype.updateAliasForLayout = function(layoutId, aliasData) {
  var layout = this._layoutsData[layoutId];
  var alias = new LayoutAlias(aliasData);
  layout.updateAlias(alias);

  //if we don't have basic information about alias then mark alias to be missing
  if (this._aliases[alias.id] == null) {
    if (this._missingAliases[alias.id] == null) {
      this._missingAliases[alias.id] = alias.id;
    }
  }
};


module.exports = MapModel;

