"use strict";

var functions = require('../../Functions');

var AbstractOverlayElement = require('./AbstractOverlayElement');

/**
 * Class representing overlay of the alias on the map relevant for a specific
 * layout.
 */
function AliasOverlay(paramObj, map, aliasData) {
  // call super constructor
  AbstractOverlayElement.call();

  // google map object associated with the alias
  this.gmapObj = paramObj;
  // AbstractCustomMap where the alias is located
  this.customMap = map;
  // original data
  this.aliasData = aliasData;

  var self = this;
  var onclick = (function() {
    var aliasOverlayData = self;
    return function() {
      customMap.openInfoWindowForAlias(aliasOverlayData.aliasData.id,
          aliasOverlayData.customMap.id);
    };
  })();
  google.maps.event.addListener(paramObj, 'click', onclick);
};

AliasOverlay.prototype = Object.create(AbstractOverlayElement.prototype);
AliasOverlay.prototype.constructor = AliasOverlay;

/**
 * Sets Google map to this {@link AliasOverlay}.
 * 
 * @param map
 *            map to set
 */
AliasOverlay.prototype.setMap = function(map) {
  this.gmapObj.setMap(map);
};

/**
 * Function used to recalculate boundaries of the {@link AliasOverlay}.
 * Boundaries define how big part of original alias is taken by this layout
 * visualization.
 * 
 * @param startX
 *            value between 0..1 defininf where should be the start on OX axis
 * @param endX
 *            value between 0..1 defininf where should be the end on OX axis
 */
AliasOverlay.prototype.setBoundsForAlias = function(startX, endX) {
  var pointA = new google.maps.Point(this.aliasData.x + startX
      * this.aliasData.width, this.aliasData.y);
  var pointB = new google.maps.Point(this.aliasData.x + endX
      * this.aliasData.width, this.aliasData.y + this.aliasData.height);
  var latLngA = this.customMap.fromPointToLatLng(pointA);
  var latLngB = this.customMap.fromPointToLatLng(pointB);

  var bounds = new google.maps.LatLngBounds();
  bounds.extend(latLngA);
  bounds.extend(latLngB);

  this.gmapObj.setBounds(bounds);
};

/**
 * Creates {@link AliasOverlay} from input data.
 * 
 * @param layoutAlias
 *            {@link LayoutAlias} for which overlay is created
 * @param aliasData
 *            {@link Alias data} of the alias
 * @param map
 *            {@link AbstractCustomMap} where overlay should be placed
 * @param startX
 *            this is the ratio on OX axis that should be use as a starting
 *            point of the overlay. For instance when there are three layouts to
 *            visualize then
 *            <ul>
 *            <li>the first layout have startX=0.0; endX=0.33333</li>
 *            <li>second layout have startX=0.33333; endX=0.66666</li>
 *            <li>the last layout have startX=0.66666; endX=1.0</li>
 *            </ul>
 * @param endX
 *            this is the ratio on OX axis that should be use as a starting
 *            point of the overlay
 * @returns {AliasOverlay}
 */
AliasOverlay.create = function(layoutAlias, aliasData, map, startX, endX) {
  var pointA = new google.maps.Point(aliasData.x + startX * aliasData.width,
      aliasData.y);
  var pointB = new google.maps.Point(aliasData.x + endX * aliasData.width,
      aliasData.y + aliasData.height);
  var latLngA = map.fromPointToLatLng(pointA);
  var latLngB = map.fromPointToLatLng(pointB);

  var bounds = new google.maps.LatLngBounds();
  bounds.extend(latLngA);
  bounds.extend(latLngB);
  var rectangle = new google.maps.Rectangle({
    fillOpacity : 0.8,
    strokeWeight : 1,
    map : map.map,
    fillColor : functions.overlayToColor(layoutAlias),
    bounds : bounds
  });
  var result = new AliasOverlay(rectangle, map, aliasData);
  return result;
};

module.exports = AliasOverlay;
